/* eslint-env jest */

const main = require("../");
const plugins = require("../plugins");
const path = require("path");

describe("Last selector of __fixtures/button/index.scss", () => {
  let cssTokenized;

  describe("preserves tokens and obeys aura rules", () => {
    beforeAll(() => {
      cssTokenized = main({
        entry: path.resolve(__dirname, "../", "__fixtures__", "index.scss"),
        getTokens: () => [
          "color-brand",
          "spacing-x-small",
          "square-icon-small-content"
        ],
        plugins: {
          sass: [plugins.mixin, plugins.value, plugins.fn, plugins.negation],
          css: [plugins.font, plugins.comment, plugins.tokenString]
        },
        info: { version: "123", sha: "abc" }
      }).cssTokenized;
    });

    it("in values", () => {
      expect(cssTokenized).toContain(
        "margin-left: token(spacingXSmall), token(squareIconSmallContent)"
      );
    });

    it("not in invalid values according to aura", () => {
      expect(cssTokenized).not.toMatch(/padding: token(spacingXSmall)/gi);
    });

    it("in negation", () => {
      expect(cssTokenized).toContain(`margin-right: token('-'+spacingXSmall);`);
    });

    it("replaces comments and adds version", () => {
      expect(cssTokenized).not.toContain("Use a neutral icon button");
      expect(cssTokenized).toContain("design-system-internal 123 (abc)");
    });
  });

  describe("preserves tokens at all costs", () => {
    beforeAll(() => {
      cssTokenized = main({
        entry: path.resolve(__dirname, "../", "__fixtures__", "index.scss"),
        getTokens: () => [
          "color-brand",
          "spacing-x-small",
          "square-icon-small-content"
        ],
        plugins: {
          sass: [plugins.mixin, plugins.operations, plugins.value, plugins.fn]
        },
        info: {},
        transform: x => x.replace(/"|\+/gi, "")
      }).cssTokenized;
    });

    it("in values", () => {
      expect(cssTokenized).toContain(
        "margin-left: spacingXSmall, squareIconSmallContent"
      );
    });

    it("in invalid values according to aura", () => {
      expect(cssTokenized).toContain("padding: spacingXSmall");
    });

    it("in includes", () => {
      expect(cssTokenized).toMatch(/width: squareIconSmallContent/gi);
    });

    it("in functions", () => {
      expect(cssTokenized).toContain("background: url(colorBrand)");
    });

    it("in negation", () => {
      expect(cssTokenized).toMatch(/margin-right: spacingXSmall/gi);
    });

    it("doesnt replace comments or add a version", () => {
      expect(cssTokenized).toContain("Use a neutral icon button");
      expect(cssTokenized).not.toContain("design-system-internal 123 (abc)");
    });
  });
});
