/* eslint-env jest */

const { runPlugins } = require('./helpers');

const valuePlugin_ = require('../value');
const valuePlugin = x => valuePlugin_(x, { transform: x => `token(${x})` });

it('ignores valid tokens not inside a block', () => {
  let actual = runPlugins(
    `
    $color: $color-brand;
  `,
    [valuePlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('ignores valid tokens not inside a value', () => {
  let actual = runPlugins(
    `
    .a { $color-brand: $blue; }
  `,
    [valuePlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('ignores invalid tokens', () => {
  let actual = runPlugins(
    `
    .a { color: $color-brand; }
  `,
    [valuePlugin([])]
  );
  expect(actual).toMatchSnapshot();
});

it('ignores valid tokens inside arguments', () => {
  let actual = runPlugins(
    `
    @mixin myMixin($color: $color-brand) {}
  `,
    [valuePlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('ignores valid tokens inside parentheses', () => {
  let actual = runPlugins(
    `
    ($color-brand * 2)
  `,
    [valuePlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('ignores valid tokens inside interpolation', () => {
  let actual = runPlugins(
    `
    .#{$color-brand} {}
  `,
    [valuePlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('replaces valid tokens', () => {
  let actual = runPlugins(
    `
    .a { color: $color-brand; }
  `,
    [valuePlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('replaces valid token within css var assignment', () => {
  const actual = runPlugins(
    `
    .a { --sds-c-icon-color-background: $color-brand; }
  `,
    [valuePlugin(['color-brand'])]
  );
  expect(actual).toMatchSnapshot();
});

it('replaces all valid tokens', () => {
  let actual = runPlugins(
    `
    .a { color: $color-brand-a $color-brand-b $color-brand-c; }
  `,
    [valuePlugin(['color-brand-a', 'color-brand-c'])]
  );
  expect(actual).toMatchSnapshot();
});

it('takes a transform', () => {
  let actual = runPlugins(
    `
    .a { color: $color-brand-a $color-brand-b $color-brand-c; }
  `,
    [valuePlugin_(['color-brand-a', 'color-brand-c'], { transform: x => x })]
  );
  expect(actual).toMatchSnapshot();
});

it('finds and replaces new-line characters in values', () => {
  let actual = runPlugins(
    `
    .slds-accordion__summary {
      display: flex;
      margin-top: calc(
        var(--slds-c-accordion-section-spacing-block-start,
        var(--sds-c-accordion-section-spacing-block-start,
        #{$spacing-small} * -1)));
      margin-right: calc(
        var(--slds-c-accordion-section-spacing-inline-end,
        var(--sds-c-accordion-section-spacing-inline-end,
        #{$spacing-small} * -1)));
      margin-bottom: calc(
        var(--slds-c-accordion-section-spacing-block-end,
        var(--sds-c-accordion-section-spacing-block-end,
        #{$spacing-small} * -1)));
      margin-left: calc(
        var(--slds-c-accordion-section-spacing-inline-start,
        var(--sds-c-accordion-section-spacing-inline-start,
        #{$spacing-small} * -1)));
    }
  `,
    [valuePlugin()]
  );
  expect(actual).toMatchSnapshot();
});

it('finds and replaces new-line characters in values with comments', () => {
  let actual = runPlugins(
    `
    circle { // 1
      stroke-width: 0.95px;
      fill: transparent;
      stroke: #ccc;
      transition: // 2
        fill 0.4s ease-in-out,
        stroke 0.4s ease-in-out;
      will-change: fill, stroke; // 3
    }
  `,
    [valuePlugin()]
  );
  expect(actual).toMatchSnapshot();
});
