const { validToken, createToken, notParents } = require("../utils");

module.exports =
  (tokens, options = {}) =>
  ($) => {
    /**
     * Replaces declaration body if the property is a css custom property and the body contains a scss interpolation
     *
     * @param {NodeWrapper} declarationNode
     * @returns {NodeWrapper}
     */
    const replaceInterpolationWithTokens = (declarationNode) => {
      const declarationProp = $(declarationNode)
        .children("property")
        .value()
        .toString();
      const declarationBody = $(declarationNode).children("value");

      if (String(declarationProp).startsWith("--")) {
        declarationBody
          .find("interpolation")
          .filter(notParents("function", $))
          .replace((n) => {
            // Replace interpolation node with tokenized child scss variable
            const { type } = n.node.value[0];
            const variableNode = $(n).children("variable").nodes[0];
            if (validToken(tokens)(variableNode)) {
              return createToken(options.transform)(variableNode);
            }
            return n;
          });
      }
      return declarationNode;
    };

    $().find("declaration").replace(replaceInterpolationWithTokens);
  };
