const _ = require('lodash');
const { stringify, parse } = require('scss-parser');

const KEYWORD_BLOCKLIST = ['use'];
const PACKAGE_BLOCKLIST = ['math'];
const FUNCTION_BLOCKLIST = ['div'];

const mathDiv = `
@function polyfill-math-div($a, $b) {
  @return $a / $b;
}
`

const polyfillFuncs = { div: parse(mathDiv)}

module.exports = (options = {}) => $ => {
  const polyfillsActivated = [];

  $()
    .find('identifier')
    .filter(identifier => PACKAGE_BLOCKLIST.includes($(identifier).value()))
    .map(identifier => {
      $(identifier.parent)
        .find('class')
        .filter(classFunction => FUNCTION_BLOCKLIST.includes($(classFunction).value()))
        .replace((n) => {
          // if we're replacing the function name with the polyfill function name,
          // then add the function name to a list to add the polyfill function to the top of the sheet
          const classFunctionValue = $(n).find("identifier").value();
          polyfillsActivated.findIndex(i => i === classFunctionValue) === -1
            && polyfillsActivated.push(classFunctionValue);
          return {
          type: 'function',
          value: `polyfill-math-${classFunctionValue}`
        }});
      $(identifier).remove();
      return identifier;
    })

  // add the functions for the polyfill function name swaps done earlier
  polyfillsActivated.forEach(polyfill => {
    $()
    .children()
    .eq(0)
    .before(polyfillFuncs[polyfill]);
  })

  $()
    .find('atkeyword')
    .filter(keyword => KEYWORD_BLOCKLIST.includes($(keyword).value()))
    .parent()
    .remove();

};
