const _ = require("lodash");
const { validToken, createToken, notParents } = require("../utils");

module.exports =
  (tokens, options = {}) =>
  ($) => {
    $()
      .find("block")
      .find("value")
      .find("variable")
      .filter(validToken(tokens))
      .filter(notParents("arguments", $))
      .filter(notParents("parentheses", $))
      .filter(notParents("interpolation", $))
      .replace(createToken(options.transform));

    // Ruby Sass has a tendency to output new-line characters to `\a`
    // in the dist CSS. This block looks for new-line characters in
    // property values and removes them, taking the extra step to
    // capture any comments that occur within the multi-line value
    // and moves them to the end of the new single-line value.
    const pattern = /\r\n|\r|\n\s{2,}/;
    let $values = $().find("declaration").find("value").has("space");

    $values.has("comment_singleline").map((value) => {
      const $value = $(value);
      const $comment = $value.find("comment_singleline");
      $value
        .parent()
        .children()
        .last()
        .after($comment.remove().nodes[0].node)
        .after({
          type: "space",
          value: " ",
        });
    });

    $values
      .find("space")
      .filter((n) => {
        return pattern.test(n.node.value);
      })
      .replace((n) => {
        let node = _.cloneDeep(n.node);
        node.value = node.value.replace(pattern, " ");
        return node;
      });
  };
